==> spread-5.0.0.54-py3-none-any.whl: spread_core/bam/handling/items/actions.py <==
import json
import os
import time

import spread_core.mqtt as mqtt
from spread_core.tools import utils
from . import conditions
from .. import const, errors


class Action:
    TYPE = None

    def __init__(self, *args, **kwargs):
        self._condition = None
        self._id = kwargs['id']
        if const.CONDITION in kwargs:
            self._condition = conditions.Condition.deserialize(kwargs[const.CONDITION])

    def serialize(self) -> dict:
        res = {const.TYPE: self.TYPE, const.ID: self.id}
        if self._condition:
            res[const.CONDITION] = self._condition.serialize()
        return res

    @property
    def id(self):
        return self._id

    @property
    def conditioned(self):
        return self._condition is not None

    @property
    def conditions(self):
        if self._condition is not None:
            return [self._condition]
        return None

    def __repr__(self):
        return self.__str__()

    def data(self, *args):
        raise NotImplementedError()

    def execute(self, *args):
        raise NotImplementedError()


class BrokerAction(Action):
    TYPE = const.BROKER_TYPE
    ANY_VALUE = '$value'

    def __init__(self, *args, **kwargs):
        super(BrokerAction, self).__init__(*args, **kwargs)
        self._topic = mqtt.of(kwargs[const.TOPIC])
        self._value = kwargs[const.VALUE]
        self.delay = kwargs.get(const.DELAY, 0)
        self.is_retained = kwargs.get(const.RETAINED, False)

    def serialize(self) -> dict:
        res = super(BrokerAction, self).serialize()
        res[const.TOPIC] = str(self._topic)
        res[const.VALUE] = self._value
        return res

    def __str__(self):
        return f'{self.entity_address.type}[{self.entity_address.id}]: "{self._command}" set {self._value}'

    @property
    def topic(self):
        return self._topic

    @topic.setter
    def topic(self, topic: mqtt.spread.topic.SpreadTopic):
        self._topic = topic

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        self._value = value

    @property
    def entity_address(self) -> mqtt.spread.address.EntityAddress:
        return self._topic.entity_address

    @property
    def _command(self):
        return self.entity_address.funit_type

    def data(self, rec_value) -> mqtt.spread.variable.Variable:
        _value = rec_value if self._value == self.ANY_VALUE else self._value
        return mqtt.spread.variable.Variable(_value)

    def execute(self, handler, rec_value=None, condition_checker=None):
        if self._condition:
            if condition_checker(self._condition.topic) != self._condition.value:
                return
        handler(self._topic, self.data(rec_value), self.is_retained)
        if self.delay > 0:
            time.sleep(self.delay/1000)


class ScenarioAction(Action):
    TYPE = const.SCENARIO_TYPE

    def __init__(self, *args, **kwargs):
        super(ScenarioAction, self).__init__(*args, **kwargs)
        file_path = kwargs[const.FILE]
        self.file = file_path
        self.actions = []

        if os.path.isfile(file_path):
            with open(self.file, 'r') as file:
                data = file.read()
            if not data:
                data = '[]'
            data = json.loads(data)
            for action_data in data:
                action = action_of_data(action_data)
                self.actions.append(action)
        else:
            raise errors.HandlingError(f'Scenarios file "{self.file}" not exist!')

    def __str__(self):
        return f'Scenario of "{self.file}"'

    @property
    def conditioned(self):
        for action in self.actions:
            if action.conditioned:
                return True
        return False

    @property
    def conditions(self):
        return [action._condition for action in self.actions if action._condition is not None]

    def execute(self, *args):
        for action in self.actions:
            action.execute(*args)


def action_of_data(data) -> Action:
    def separator(subclass): return issubclass(subclass, Action) and subclass.TYPE == data[const.TYPE]
    action_class = utils.get_subclass(Action, separator=separator)
    return action_class(**data)

==> spread-5.3.0.5-py3-none-any.whl: spread_core/bam/handling/items/actions.py <==
import json
import os
import time

import spread_core.mqtt as mqtt
from spread_core.tools import utils
from . import conditions
from .. import const, errors


class Action:
    TYPE = None

    def __init__(self, *args, **kwargs):
        self._condition = None
        self._id = kwargs['id']
        if const.CONDITION in kwargs:
            self._condition = conditions.Condition.deserialize(kwargs[const.CONDITION])

    def serialize(self) -> dict:
        res = {const.TYPE: self.TYPE, const.ID: self.id}
        if self._condition:
            res[const.CONDITION] = self._condition.serialize()
        return res

    @property
    def id(self):
        return self._id

    @property
    def conditioned(self):
        return self._condition is not None

    @property
    def conditions(self):
        if self._condition is not None:
            return [self._condition]
        return None

    def __repr__(self):
        return self.__str__()

    def data(self, *args):
        raise NotImplementedError()

    def execute(self, *args, **kwargs):
        raise NotImplementedError()


class BrokerAction(Action):
    TYPE = const.BROKER_TYPE
    ANY_VALUE = '$value'

    def __init__(self, *args, **kwargs):
        super(BrokerAction, self).__init__(*args, **kwargs)
        self._topic = mqtt.of(kwargs[const.TOPIC])
        self._value = kwargs[const.VALUE]
        self.delay = kwargs.get(const.DELAY, 0)
        self.is_retained = kwargs.get(const.RETAINED, False)

    def serialize(self) -> dict:
        res = super(BrokerAction, self).serialize()
        res[const.TOPIC] = str(self._topic)
        res[const.VALUE] = self._value
        return res

    def __str__(self):
        return f'{self.entity_address.type}[{self.entity_address.id}]: "{self._command}" set {self._value}'

    @property
    def topic(self):
        return self._topic

    @topic.setter
    def topic(self, topic: mqtt.spread.topic.SpreadTopic):
        self._topic = topic

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        self._value = value

    @property
    def entity_address(self) -> mqtt.spread.address.EntityAddress:
        return self._topic.entity_address

    @property
    def _command(self):
        return self.entity_address.funit_type

    def data(self, rec_value, session_id) -> mqtt.spread.variable.Variable:
        _value = rec_value if self._value == self.ANY_VALUE else self._value
        return mqtt.spread.variable.Variable(_value, session=session_id)

    def execute(self, handler, rec_value=None, condition_checker=None, session_id=None):
        if self._condition:
            if condition_checker(self._condition.topic) != self._condition.value:
                return
        handler(self._topic, self.data(rec_value, session_id=session_id), self.is_retained)
        if self.delay > 0:
            time.sleep(self.delay/1000)


class ScenarioAction(Action):
    TYPE = const.SCENARIO_TYPE

    def __init__(self, *args, **kwargs):
        super(ScenarioAction, self).__init__(*args, **kwargs)
        file_path = kwargs[const.FILE]
        self.file = file_path
        self.actions = []

        if os.path.isfile(file_path):
            with open(self.file, 'r') as file:
                data = file.read()
            if not data:
                data = '[]'
            data = json.loads(data)
            for action_data in data:
                action = action_of_data(action_data)
                self.actions.append(action)
        else:
            raise errors.HandlingError(f'Scenarios file "{self.file}" not exist!')

    def __str__(self):
        return f'Scenario of "{self.file}"'

    @property
    def conditioned(self):
        for action in self.actions:
            if action.conditioned:
                return True
        return False

    @property
    def conditions(self):
        return [action._condition for action in self.actions if action._condition is not None]

    def execute(self, *args, **kwargs):
        for action in self.actions:
            action.execute(*args, **kwargs)


def action_of_data(data) -> Action:
    def separator(subclass): return issubclass(subclass, Action) and subclass.TYPE == data[const.TYPE]
    action_class = utils.get_subclass(Action, separator=separator)
    return action_class(**data)
